@@ -17,7 +17,7 @@ This is made up of:
 # parse_keqv_list, parse_http_list
 # GopherHandler (haven't used gopher for a decade or so...)
 
-import unittest, StringIO, os, sys, UserDict, httplib
+import unittest, StringIO, os, sys, UserDict, httplib, warnings
 
 import mechanize
 
@@ -28,11 +28,16 @@ from mechanize import HTTPRedirectHandler, HTTPRequestUpgradeProcessor, \
      HTTPCookieProcessor, HTTPRefererProcessor, \
      HTTPErrorProcessor, HTTPHandler
 from mechanize import OpenerDirector, build_opener, urlopen, Request
+from mechanize._util import hide_deprecations, reset_deprecations
 
 ## from logging import getLogger, DEBUG
 ## l = getLogger("mechanize")
 ## l.setLevel(DEBUG)
 
+class AlwaysEqual:
+    def __cmp__(self, other):
+        return 0
+
 class MockOpener:
     addheaders = []
     def open(self, req, data=None):
@@ -259,6 +264,20 @@ class OpenerDirectorTests(unittest.TestCase):
                 if expected_args[j] is not Unknown:
                     self.assert_(args[j] == expected_args[j])
 
+    def test_http_error_raised(self):
+        # should get an HTTPError if an HTTP handler raises a non-200 response
+        # XXX it worries me that this is the only test that excercises the else
+        # branch in HTTPDefaultErrorHandler
+        from mechanize import _response
+        o = mechanize.OpenerDirector()
+        o.add_handler(mechanize.HTTPErrorProcessor())
+        o.add_handler(mechanize.HTTPDefaultErrorHandler())
+        class HTTPHandler(AbstractHTTPHandler):
+            def http_open(self, req):
+                return _response.test_response(code=302)
+        o.add_handler(HTTPHandler())
+        self.assertRaises(mechanize.HTTPError, o.open, "http://example.com/")
+
     def test_processors(self):
         # *_request / *_response methods get called appropriately
         o = OpenerDirector()
@@ -656,21 +675,52 @@ class HandlerTests(unittest.TestCase):
         self.assert_(req.unredirected_hdrs["Referer"] == ref)
 
     def test_errors(self):
+        from mechanize import _response
         h = HTTPErrorProcessor()
         o = h.parent = MockOpener()
 
-        url = "http://example.com/"
-        req = Request(url)
+        req = Request("http://example.com")
         # 200 OK is passed through
-        r = MockResponse(200, "OK", {}, "", url)
+        r = _response.test_response()
         newr = h.http_response(req, r)
         self.assert_(r is newr)
         self.assert_(not hasattr(o, "proto"))  # o.error not called
         # anything else calls o.error (and MockOpener returns None, here)
-        r = MockResponse(201, "Created", {}, "", url)
+        r = _response.test_response(code=201, msg="Created")
         self.assert_(h.http_response(req, r) is None)
         self.assert_(o.proto == "http")  # o.error called
-        self.assert_(o.args == (req, r, 201, "Created", {}))
+        self.assert_(o.args == (req, r, 201, "Created", AlwaysEqual()))
+
+    def test_raise_http_errors(self):
+        # HTTPDefaultErrorHandler should raise HTTPError if no error handler
+        # handled the error response
+        from mechanize import _response
+        h = mechanize.HTTPDefaultErrorHandler()
+
+        url = "http://example.com"; code = 500; msg = "Error"
+        request = mechanize.Request(url)
+        response = _response.test_response(url=url, code=code, msg=msg)
+
+        # case 1. it's not an HTTPError
+        try:
+            h.http_error_default(
+                request, response, code, msg, response.info())
+        except mechanize.HTTPError, exc:
+            self.assert_(exc is not response)
+            self.assert_(exc.fp is response)
+        else:
+            self.assert_(False)
+
+        # case 2. response object is already an HTTPError, so just re-raise it
+        error = mechanize.HTTPError(
+            url, code, msg, "fake headers", response)
+        try:
+            h.http_error_default(
+                request, error, code, msg, error.info())
+        except mechanize.HTTPError, exc:
+            self.assert_(exc is error)
+        else:
+            self.assert_(False)
 
     def test_robots(self):
         # XXX useragent
@@ -756,6 +806,41 @@ class HandlerTests(unittest.TestCase):
         h.http_request(req)
         self.assert_(rfpc.calls == [])
 
+    def test_redirected_robots_txt(self):
+        # redirected robots.txt fetch shouldn't result in another attempted
+        # robots.txt fetch to check the redirection is allowed!
+        import mechanize
+        from mechanize import build_opener, HTTPHandler, \
+             HTTPDefaultErrorHandler, HTTPRedirectHandler, \
+             HTTPRobotRulesProcessor
+
+        class MockHTTPHandler(mechanize.BaseHandler):
+            def __init__(self):
+                self.requests = []
+            def http_open(self, req):
+                import mimetools, httplib, copy
+                from StringIO import StringIO
+                self.requests.append(copy.deepcopy(req))
+                if req.get_full_url() == "http://example.com/robots.txt":
+                    hdr = "Location: http://example.com/en/robots.txt\r\n\r\n"
+                    msg = mimetools.Message(StringIO(hdr))
+                    return self.parent.error(
+                        "http", req, test_response(), 302, "Blah", msg)
+                else:
+                    return test_response("Allow: *", [], req.get_full_url())
+
+        hh = MockHTTPHandler()
+        hdeh = HTTPDefaultErrorHandler()
+        hrh = HTTPRedirectHandler()
+        rh = HTTPRobotRulesProcessor()
+        o = build_test_opener(hh, hdeh, hrh, rh)
+        o.open("http://example.com/")
+        self.assertEqual([req.get_full_url() for req in hh.requests],
+                         ["http://example.com/robots.txt",
+                          "http://example.com/en/robots.txt",
+                          "http://example.com/",
+                          ])
+
     def test_cookies(self):
         cj = MockCookieJar()
         h = HTTPCookieProcessor(cj)
@@ -773,7 +858,11 @@ class HandlerTests(unittest.TestCase):
         self.assert_(cj.ec_u == False)
 
     def test_seekable(self):
-        h = SeekableProcessor()
+        hide_deprecations()
+        try:
+            h = SeekableProcessor()
+        finally:
+            reset_deprecations()
         o = h.parent = MockOpener()
 
         req = mechanize.Request("http://example.com/")
@@ -788,25 +877,28 @@ class HandlerTests(unittest.TestCase):
         self.assert_(hasattr(newr, "seek"))
 
     def test_http_equiv(self):
+        from mechanize import _response
         h = HTTPEquivProcessor()
         o = h.parent = MockOpener()
 
-        req = Request("http://example.com/")
-        r = MockResponse(
-            200, "OK",
-            http_message({"Foo": "Bar",
-                          "Content-type": "text/html",
-                          "Refresh": "blah"}),
-            '<html><head>'
-            '<meta http-equiv="Refresh" content="spam&amp;eggs">'
-            '</head></html>',
-            "http://example.com/"
-            )
+        data = ('<html><head>'
+                '<meta http-equiv="Refresh" content="spam&amp;eggs">'
+                '</head></html>'
+                )
+        headers = [("Foo", "Bar"),
+                   ("Content-type", "text/html"),
+                   ("Refresh", "blah"),
+                   ]
+        url = "http://example.com/"
+        req = Request(url)
+        r = _response.make_response(data, headers, url, 200, "OK")
         newr = h.http_response(req, r)
-        headers = newr.info()
-        self.assert_(headers["Foo"] == "Bar")
-        self.assert_(headers["Refresh"] == "spam&eggs")
-        self.assert_(headers.getheaders("Refresh") == ["blah", "spam&eggs"])
+
+        new_headers = newr.info()
+        self.assertEqual(new_headers["Foo"], "Bar")
+        self.assertEqual(new_headers["Refresh"], "spam&eggs")
+        self.assertEqual(new_headers.getheaders("Refresh"),
+                         ["blah", "spam&eggs"])
 
     def test_refresh(self):
         # XXX test processor constructor optional args
